<?php
/* 
 * This widget creates a multilevel drop down menu
 * The widget is based in the dropdown-menu :
 * http://www.leigeber.com/2008/11/drop-down-menu/
 *
 * The widget takes 3 partameters:
 * stylesheet : the stylesheet to use
 * menuID : the menu id
 * menu : The menu data which is an array of arrays.
 * Eash array is a menu item with the following parameters:
 * label : The label to display in the menu
 * url : The url can be a string e.g. "url"=>"/product/create" , an array
 * e.g. "url"=>array("/product/show",array("id"=>3)) or an empty string
 * e.g. "url"=>"" for a menu without a link
 * disabled : Whether the link is disabled or not, default is false
 * visible : Whether the menu item is visible or not. Default is true.
 * If a menu is not visible all of it's submenus are not visible too
 * 
 */

class SMenu extends CWidget {
  /**
   * The menu items' data
   * @var array
   */
  private $_menu;
  /**
   * The stylesheet to use
   * @var string
   */
  public $stylesheet = "menu_default.css";
  /**
   * The id of the menu
   * @var string
   */
  public $menuID = "menu";

  /**
   * Sets the menu data
   * @param array $menu
   *
   */
  public function setMenu($menu) {
    if(is_array($menu)) {
      $this->_menu = $menu;
    } else {
      throw new CException("Menu must be an array");
    }
  }

  /**
   * Execute the widget
   */
  public function run() {
    if(!isset ($this->_menu) || $this->_menu == array()) {
      throw new CException("Menu is not set or it's empty");
    }
    $this->registerClientScripts();
    $this->createMarkup();
  }
  /**
   * Registers the clientside widget files (css & js)
   */
  public function registerClientScripts() {
  //Yii::app()->clientScript->registerCoreScript('jquery');
  // Get the resources path
    $resources = dirname(__FILE__).DIRECTORY_SEPARATOR.'resources';

    // publish the files
    $baseUrl = Yii::app()->assetManager->publish($resources);

    //Debug : publish style in every request
    //Yii::app()->assetManager->publish($resources.'/'.$this->stylesheet);
    // register the files
    Yii::app()->clientScript->registerScriptFile($baseUrl.'/menu.js');
    Yii::app()->clientScript->registerCssFile($baseUrl.'/'.$this->stylesheet);
  }

  /**
   * Creates the html markup needed by the widget
   */
  public function createMarkUp() {
    echo "<ul id='".$this->menuID."' class='menu'>\n";
    $this->_createMenu($this->_menu);
    echo "</ul>\n";
    echo "<script type='text/javascript'>\n";
    echo "var ".$this->menuID."=new menu.dd('".$this->menuID."');\n";
    echo $this->menuID.".init('".$this->menuID."','menuhover');\n";
    echo "</script>\n";
  }

  /**
   * Creates the menu unordered list
   * @param array $menu : The menu array
   * @param if we're on a sub menu or not $rec
   */
  private function _createMenu($menu,$sub = false) {
    if(is_array($menu)) {
    //CVarDumper::dump($data, 5, true);
    //exit();
      foreach ($menu as $data) {
      // If in top menu set class topline
        if(!$sub) {
          $liClass= "topline";
        }
        // If there's a menu item to display
        if($this->_isMenuItem($data)) {
          $label = $data["label"];
          $href = $this->_createUrl($data);
          if (!$this->hasChild($data)) {
            $class = $href=="" ?  "disabled" : "menulink";
            echo "<li class='$liClass'><a class='".$class."' $href >".$label."</a></li>\n";
          } else {
            $class = $sub ? "sub" : "menulink";
            $class .= $href=="" ?  " disabled" : " menulink";
            echo "<li class='$liClass' ><a class='$class' $href>".$label."</a>\n";
            echo "<ul>\n";
            echo $this->_createMenu($data,true);
            echo "</ul>\n";
          }
        }
      }
    }
  }

  /**
   * Checks if there's a menu item to display
   * $data must be an array with a label key
   * and if the key visible is set it must be true
   * @param array $data
   * @return true if there's a menu item
   */
  private function _isMenuItem($data) {
    if(is_array($data) &&
        $data['label'] &&
        (!isset($data["visible"])
        || !$data["visible"]==false)
    ) {

      return true;
    }
    return false;

  }

  /**
   * Create the url link
   * @param array $data
   */
  private function _createUrl($data) {
  // If url is an array or a non blank string and it's not disabled
    if(($data['url'][0]!= "" || $data['url'] != "") && !$data["disabled"]) {
    // If there are parameters get them
      if(is_array($data['url'][1])) {
        $params = $data['url'][1];
      } else {
        $params = array();
      }
      // Create the url
      $href = "href='".$this->controller->createUrl(
          is_array($data['url']) ? $data['url'][0] : $data['url']
          ,$params)."'";
    } else {
      $href="";
    }
    return $href;
  }

  /**
   * Checks if this menu array has a submenu
   * @param array $arr
   * @return true if there's a submenu
   */
  private function hasChild($arr) {
    if(!is_array($arr)) {
      return false;
    }
    foreach ($arr as $title=>$value) {
      if(!$title == "url") {
        if(is_array($value)) {
          return true;
        }
      }
    }
    return false;
  }
}
?>
